#!/usr/bin/python

# Expects a query and uses it to access the doaj database 

import sys, getopt
import urllib
import re
import requests
import json
import pandas as pd 

def parse_query(query_file, page_size=100, start_page=1):
    with open(query_file, 'r') as query_handle:
        query_content = query_handle.read()
        comment, plain_query, further = query_content.split("::", 2)
        plain_query = re.sub(" +\n|\n", " ", plain_query)
        plain_query = re.sub("^ +| +$", "", plain_query)
        encoded_query = urllib.quote_plus(plain_query, safe="()")
        # DOAJ does not like encoding spaces with '+' - it has to be escaped per '%20'
        encoded_query = re.sub("\+", "%20", encoded_query)
        base = "https://doaj.org/api/v1/search/articles/"
        options = "?page=" + str(start_page) + "&pageSize=" + str(page_size)
        complete_url = base + encoded_query + options
        print "Querying the results for:"
        print complete_url
        return complete_url

def execute_query(query_url):
    papers = []
    headers = {
        'Accept': 'application/json'
    }
    print "Loading results"
    response = requests.get(query_url, headers=headers)
    if response.status_code != 200:
        print "Received HTTP-Statuscode other than 200 - Aborting" + str(response.status_code)
        if response.status_code == 400:
            print "Error is probably in the query syntax."
        sys.exit(1)
    else:
        json_data = json.loads(response.text)
        print "The number of articles matching the query is: " + str(json_data['total'])
        papers = papers + json_data['results']
        while 'next' in json_data:
            print "Getting further results.."
            response = requests.get(json_data['next'], headers=headers)
            json_data = json.loads(response.text)
            papers = papers + json_data['results']
            print "Currently at: " + str(len(papers))
        return papers

def parse_result(result):
    df = pd.DataFrame(index = [], columns=['doi', 'title', 'abstract', 'authors', 'journal/conference'])
    for paper in result:
        info = paper['bibjson']
        title = info['title']
        journal = info['journal']['title']
        abstract = info['abstract']
        extracted_authors = []
        for author in info['author']:
            extracted_authors.append(author['name'])
        authors = ','.join(extracted_authors)
        doi = ""
        for identifier in info['identifier']:
            if identifier['type'] == "doi":
                doi = identifier['id']
        if not doi:
            if "link" in info:  
                for link in info['link']:
                    if "doi.org" in link['url']:
                        doi = link['url'].split("doi.org/")[1]
                        
        df = df.append({'doi' : doi, 'title' : title, 'abstract' : abstract, 'authors' : authors, 'journal/conference' : journal}, ignore_index=True)
    return df

def retrieve_results(query_file):
    query = parse_query(query_file)
    result = execute_query(query)
    csv = parse_result(result)
    out_file = "doaj_query_result"
    print "Writing output to " + out_file
    csv.to_csv(out_file + '.csv', index = False, encoding='utf-8')

def main(argv):
    query_source = ''
    try:
        opts, args = getopt.getopt(argv,"hi:",["ifile="])
    except getopt.GetoptError:
        print 'Usage: doaj_access_script -i <doaj-query>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'Usage: doaj_access_script -i <doaj-query>'
            sys.exit()
        elif opt in ("-i", "--ifile"):
            query_source = arg
    if not query_source:
        print 'Valid input file expected. Usage: doaj_access_script -i <doaj-query>' 
        sys.exit(2)

    retrieve_results(query_source)

if __name__ == "__main__":
   main(sys.argv[1:])
