#!/usr/bin/python

# Expects a query and uses it to access the scopus database 

import sys, getopt
import urllib
import re
import requests
import json
import pandas as pd 
import os.path

def parse_query(query_file, page_size=100, start_count=0):
    if not os.path.isfile("key"):
        print "To access Scopus an API key is needed: the script assumes it can be found as plain text in the same directory under the filename 'key'. To get a key visit https://dev.elsevier.com/"
        sys.exit(1)
    with open(query_file, 'r') as query_handle, open('key', 'r') as key_handle:
        key = key_handle.read()
        query_content = query_handle.read()
        comment, plain_query, further = query_content.split("::", 2)
        plain_query = re.sub(" +\n|\n", " ", plain_query)
        plain_query = re.sub("^ +| +$", "", plain_query)
        encoded_query = urllib.quote_plus(plain_query, safe="()*")
        # Scopus standard query encoding escapes spaces with '%20' - following this style:
        encoded_query = re.sub("\+", "%20", encoded_query)
        base = "https://api.elsevier.com/content/search/scopus?start=" + str(start_count) + "&count=" + str(page_size) + "&query="
        options = "&apiKey=" + key
        complete_url = base + encoded_query + options
        print "Querying the results for:"
        print complete_url
        return complete_url

def execute_query(query_url):
    papers = []
    headers = {
        'Accept': 'application/json'
    }
    print "Loading results"
    response = requests.get(query_url, headers=headers)
    if response.status_code != 200:
        print "Received HTTP-Statuscode other than 200 - Aborting: " + str(response.status_code)
        if response.status_code == 400:
            print "Error is probably in the query syntax."
        sys.exit(1)
    else:
        json_data = json.loads(response.text)
        result = json_data['search-results']
        print "The number of articles matching the Query is: " + str(result['opensearch:totalResults'])
        print "Loading " + str(result['opensearch:itemsPerPage']) + " articles at a time."
        papers = papers + result['entry']
        links = result['link']
        resume = False
        for link in links:
            if 'next' == link['@ref']: 
                next_url = link['@href']
                resume = True  
        while resume:
            print "Getting further results.."
            response = requests.get(next_url, headers=headers)
            json_data = json.loads(response.text)
            result = json_data['search-results']
            papers = papers + result['entry']
            print "Currently at: " + str(len(papers))
            links = result['link']
            resume = False
            for link in links:
                if 'next' == link['@ref']: 
                    print "Loading further data.."
                    next_url = link['@href']
                    resume = True
        print "\n" + str(len(papers)) + " articles were loaded. "
        return papers

def parse_result(result):
    df = pd.DataFrame(index = [], columns=['doi', 'title', 'abstract', 'authors', 'journal/conference'])
    for paper in result:
        title, journal, abstract, authors, doi = "","","","","" # Scopus does not allow direct abstract Download - need a second API for that
        if 'dc:title' in paper:
            title = paper['dc:title']
        if 'prism:publicationName' in paper:
            journal = paper['prism:publicationName']
        if 'dc:creator' in paper:
            authors = paper['dc:creator']
        if 'prism:doi' in paper:
            doi = paper['prism:doi']
        df = df.append({'doi' : doi, 'title' : title, 'abstract' : abstract, 'authors' : authors, 'journal/conference' : journal}, ignore_index=True)
    return df

def retrieve_results(query_file):
    query = parse_query(query_file)
    result = execute_query(query)
    csv = parse_result(result)
    out_file = "scopus_query_result"
    print "Writing output to " + out_file
    csv.to_csv(out_file + '.csv', index = False, encoding='utf-8')

def main(argv):
    query_source = ''
    try:
        opts, args = getopt.getopt(argv,"hi:",["ifile="])
    except getopt.GetoptError:
        print 'Usage: scopus_access_script.py -i <scopus-query>'
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print 'Usage: scopus_access_script.py -i <scopus-query>'
            sys.exit()
        elif opt in ("-i", "--ifile"):
            query_source = arg
    if not query_source:
        print 'Valid input file expected. Usage: scopus_access_script.py -i <scopus-query>' 
        sys.exit(2)

    retrieve_results(query_source)

if __name__ == "__main__":
   main(sys.argv[1:])